#!/bin/bash

# Version 16/11/2012
clear
echo "Installing BuDuScRiPt plugin:"

################################################################################
script_path=$(echo "${0}" | sed 's@/install.sh@@')
update_file=${script_path}/buduscript_update.dat
current_dir=$(pwd)

################################################################################
# Prova a creare lo swap-file utilizzato come semaforo binario in "/tmp".
echo " * Verifica dei permessi della cartella temporanea ..."
cd /tmp
if [ "$(pwd)" != "/tmp" ]; then
    echo "ATTENZIONE: Non e' stato possibile accedere alla cartella '/tmp'."
    echo "Possibili cause:"
    echo " - Mancano i permessi per l'utente corrente"
    echo "   \_ SOLUZIONE: Verificare e correggere i permessi della directory '/tmp'."
    
    exit 1;
fi

dd if=/dev/zero of=buduscript.swapfile bs=1024 count=1000 > /dev/null
if [ ! -f buduscript.swapfile ]; then
    echo "ATTENZIONE: Non e' stato possibile creare lo swap-file."
    echo "Possibili cause:"
    echo " - Mancano i permessi in scrittura nella cartella '/tmp'."
    echo "   \_ SOLUZIONE: Verificare e correggere i permessi della directory corrente."
    echo " - Non e' disponibile sufficiente spazio su disco (1Mb)."
    echo "   \_ SOLUZIONE: Liberare spazio."
    
    exit 1;
fi
rm -f buduscript.swapfile
cd "${current_dir}"
echo "fatto."

################################################################################
if [ "${1}" == "2" ]; then
    cd "${HOME}/.xchat2/buduscript"
    current_dir=$(pwd)
fi

################################################################################
if [ ! -f version.c ]; then
    printf "ERRORE FATALE: Nella cartella corrente (%s) non sono stati trovati i files del plugin.\n" "$(pwd)"
    printf "Cambiare la cartella corrente e lanciare './install.sh'\n\n"
    
    exit 2;
fi

################################################################################
if [ "$(echo "${script_path} ${current_dir}" | grep -F "/.xchat2/")" != "" ]; then
    response=2
else
    if [ "${1}" == "" ]; then
	echo "Quale Versione di XChat e' installata ? (1/2)"
	read response
    else
	response=${1}
    fi
fi

################################################################################
if [ "${HOME}" == "/root" ]; then
    echo "ERRORE FATALE: NON PUOI installare BuDuScRiPt COME UTENTE root !"
    echo "ABORT."

    exit 255
fi

################################################################################
if [ "${response}" == "1" ]; then
    echo -n "Creazione del Simbolic Link per XChat 1.x ..."
    xchat_dir=${HOME}/.xchat
else
    echo -n "Creazione del Simbolic Link per XChat 2.x ..."
    xchat_dir=${HOME}/.xchat2
fi
echo "fatto."

################################################################################
echo -n " * Verifica della caretlla ${xchat_dir} ..."
if [ ! -d "${xchat_dir}" ]; then
        echo "OOPS, ${xchat_dir} non trovata !"
        exit 0;
fi
echo "fatto."

################################################################################
# Prova a creare lo swap-file utilizzato come semaforo binario.
echo " * Verifica dei permessi in scrittura ..."
dd if=/dev/zero of=buduscript.swapfile bs=1024 count=1000 > /dev/null
if [ ! -f buduscript.swapfile ]; then
    echo "ATTENZIONE: Non e' stato possibile creare lo swap-file."
    echo "Possibili cause:"
    echo " - Mancano i permessi in scrittura"
    echo "   \_ SOLUZIONE: Verificare e correggere i permessi della directory corrente."
    echo " - Non e' disponibile sufficiente spazio su disco (1Mb)."
    echo "   \_ SOLUZIONE: Liberare spazio."
    
    exit 1;
fi
rm -f buduscript.swapfile
echo "fatto."

################################################################################
echo -n " * Verifica della cartella ${xchat_dir}/buduscript ..."
if [ ! -d "${xchat_dir}/buduscript" ]; then
        echo "OOPS, ${xchat_dir}/buduscript non trovata !"
	echo -n " * Creazione automatica ..."
	mkdir "${xchat_dir}/buduscript"
	if [ ! -d "${xchat_dir}/buduscript" ]; then
	        echo "OOPS, ${xchat_dir}/buduscript non trovata, ancora !"
		echo "Problema di permessi ? :-/"
		exit 0;
	fi
fi
echo "fatto."

################################################################################
echo -n " * Verifica della cartella ${xchat_dir}/buduscript/asciiart ..."
if [ ! -d "${xchat_dir}/buduscript/asciiart" ]; then
        echo "OOPS, ${xchat_dir}/buduscript/asciiart non trovata !"
	echo -n " * Creazione automatica ..."
	mkdir "${xchat_dir}/buduscript/asciiart"
	if [ ! -d "${xchat_dir}/buduscript/asciiart" ]; then
	        echo "OOPS, ${xchat_dir}/buduscript/asciiart non trovata, ancora !"
		echo "Problema di permessi ? :-/"
		exit 0;
	fi
fi
echo "fatto."

################################################################################
echo -n " * Verifica della cartella ${xchat_dir}/buduscript/messages  ..."
if [ ! -d "${xchat_dir}/buduscript/messages" ]; then
        echo "OOPS, ${xchat_dir}/buduscript/messages non trovata !"
	echo -n " * Creazione automatica ..."
	mkdir "${xchat_dir}/buduscript/messages"
	if [ ! -d "${xchat_dir}/buduscript/messages" ]; then
	        echo "OOPS, ${xchat_dir}/buduscript/messages non trovata, ancora !"
		echo "Problema di permessi ? :-/"
		exit 0;
	fi
fi
echo "fatto."

################################################################################
echo -n " * Verifing ${xchat_dir}/buduscript/Icons directory ..."
if [ ! -d "${xchat_dir}/buduscript/Icons" ]; then
        echo "OOPS, ${xchat_dir}/buduscript/Icons don't found !"
	echo -n " * Create it ..."
	mkdir "${xchat_dir}/buduscript/Icons"
	if [ ! -d "${xchat_dir}/buduscript/Icons" ]; then
	        echo "OOPS, ${xchat_dir}/buduscript/Icons don't found !"
		echo "Permission problem ? :-/"
		exit 0;
	fi
fi
echo "fatto."

################################################################################
echo -n " * Verifica della cartella ${HOME}/tmp ..."
if [ ! -d "${HOME}/tmp" ]; then
        echo "OOPS, ${HOME}/tmp non trovata !"
	echo -n " * Creazione automatica ..."
	mkdir "${HOME}/tmp"
	if [ ! -d "${HOME}/tmp" ]; then
	        echo "OOPS, ${HOME}/tmp non trovata, ancora !"
		echo "Problema di permessi ? :-/"
		exit 0;
	fi
fi
echo "fatto."

################################################################################
if [ "$(pwd)" != "${xchat_dir}/buduscript" ] && [ "$(dirname ${0})" != "${xchat_dir}/buduscript" ]; then
    echo -n " * Copia dei files di BuDuScRiPt nella cartella di installazione ..."
    cp -fv * ${xchat_dir}/buduscript
    cp -fv asciiart/* ${xchat_dir}/buduscript/asciiart
    cp -fv messages/* ${xchat_dir}/buduscript/messages
    cp -fv Icons/* ${xchat_dir}/buduscript/Icons
    echo -n "fatto."
fi

################################################################################
echo " * Controllo della cartella di installazione"
cd ${xchat_dir}/buduscript
echo -n " * Verifica del plugin ( budus.so ) ..."
if [ ! -f budus.so ]; then
        echo "OOPS, budus.so non trovato nella cartella corrente ($pwd) !"
	echo -n "Rimozione del link ..."
	rm -f ../budus.so && echo "fatto."

        exit 0;
fi
echo "fatto."

################################################################################
current_dir=$(pwd)
if [ -f "${xchat_dir}/budus.so" ]; then
        echo " * Rimozione del vecchio budus.so ..."
        rm -fv "${xchat_dir}/budus.so"
        echo "... fatto."
fi 

################################################################################
echo " * Verifica della Compatibilita' Hardware"
system_type=$(file /sbin/init | awk '{print $3}')
plugin_type=$(file budus.so | awk '{print $3}')

if [ "${system_type}" != "${plugin_type}" ]; then
    echo "*** WARNING *** HARDWARE INCOMPATIBILITY DETECTED ***"
    echo "Il sistema operativo usato e' $(uname -a) e utilizza software ${system_type}, mentre il plugin e' per sistemi ${plugin_type}."
    echo "*** WARNING *** HARDWARE INCOMPATIBILITY DETECTED ***"
else
    echo "... fatto."
fi

################################################################################
echo " * Verifica della Compatibilita' Software"

xchat_bin_path=$(type xchat | awk '{print $4}' | sed 's@[()]@@g')
if test "x${xchat_bin_path}" == x ; then
    xchat_bin_path=$(type xchat | awk '{print $3}' | sed 's@[()]@@g')
fi
if test "x${xchat_bin_path}" != x ; then
    echo "XChat trovato: '${xchat_bin_path}'"
    xchat_version=$(${xchat_bin_path} --version | awk '{print $2}')
    if test "x${xchat_version}" != x ; then
	check_plugin_xchat_version=$(grep -F ${xchat_version} budus.so)
	
	echo "Pacchetto del plugin installato:"
	cat ${update_file}
	
	if test "x${check_plugin_xchat_version}" == x ; then
	    echo "*** WARNING *** SOFTWARE INCOMPATIBILITY DETECTED ***"
	    echo "La versione installata di XChat e' la ${xchat_version} ma il plugin risulta per un'altra versione !"
	    echo "*** WARNING *** SOFTWARE INCOMPATIBILITY DETECTED ***"
	else
	    echo "... fatto (XChat[${xchat_version}] == BuDuScRiPt[${check_plugin_xchat_version}])."
	fi
    else
	echo "*** WARNING *** XCHAT VERSION DON'T FOUND ***"
	echo "Impossibile determinare la versione di XChat in uso."
	echo "*** WARNING *** XCHAT VERSION DON'T FOUND ***"
    fi
else
    echo "*** WARNING *** XCHAT DON'T FOUND ***"
    echo "Impossibile individuare XChat."
    echo "*** WARNING *** XCHAT DON'T FOUND ***"
fi

################################################################################
echo " * Verifica della Compatibilita' del Sistema"

bash_check=$(bash --version 2> /dev/null | grep -iF 'GNU bash')
if [ "${bash_check}" == "" ]; then
    echo "*** WARNING *** BASH DON'T FOUND ***"
else
    echo "... fatto (BASH: ${bash_check})."
fi

################################################################################
echo " * Verifica Permessi dei File"

buduscript_binary_file_list="udaplaysound
udaregexp
udapaste
budus.so"

IFS_OLD=${IFS}
IFS=$'\n'
for file in ${buduscript_binary_file_list}; do
    [ ! -x "${file}" ] && echo "ERROR (NON ESEGUIBILE): ${file}" && permission_error=1
done

for file in $(find "${xchat_dir}/buduscript/" -type f -iname '*.sh'); do
    [ ! -x "${file}" ] && echo "ERROR (NON ESEGUIBILE): ${file}" && permission_error=1
done

for file in $(find "${xchat_dir}/buduscript/" -type f); do
    [ ! -r "${file}" ] && echo "ERROR (NON ESEGUIBILE)  : ${file}" && permission_error=1
done
IFS=${IFS_OLD}

if [ ${permission_error:-0} == 1 ]; then
    echo "*** ATTENZIONE *** PROBLEMA DI PERMESSI ***"
else
    echo "... fatto."
fi

################################################################################
echo " * Installazione di BuDuScRiPt plugin"
if [ ! -f "${xchat_dir}/buduscript/budus_first" ]; then
    ln -sv "${xchat_dir}/buduscript/buduscript_script__first.sh" "${xchat_dir}/buduscript/budus_first"
fi
ln -vs "${current_dir}/budus.so" "${xchat_dir}/budus.so"
echo "fatto."

################################################################################
printf 'Se e'\'' la prima installazione - e non un aggiornamento - digitare il seguente comando in XChat: /load .xchat2/budus.so\n'

################################################################################
